`smooth.arma` <-
function(obj,  #Objeto da Classe arma
                        bairro,                 #Bairro na forma "BT", "DI", etc
			                  data=1,                 #Posicao da data - veja names(resp), ou Data no formato "YYYY-MM"
			                  grau=100,               #Grau do Grid
			                  log=TRUE,               #Usar logaritmo
		                    points=FALSE,           #Plotar pontos
                        contour=FALSE,          #Contorno
			                  method=c("loess", "gam", "akima", "tps", "mba","lm", "glm"), #Metodo
                        plot=TRUE,              #Plotar o grafico
                        col=terrain.colors(20), #Cor
                        cext = 1,               #Tamanho do Ttulo
                        showTitle=TRUE,         #Mostra Ttulo
                        statistics=FALSE,       #Retornar Anlises do model
                        trunc=FALSE,            #Truncar Preditos
                        warn=FALSE,              #Alerta para nmeros acima do normal
                        media=c(6.027562,866.2124),#Medias c/s log para todos os dados
                        desv=c(1.50505, 1087.343),#Desvios padroes c/s log para todos os dados
                        conf=0.95,              #Quantil para definir LSC
                        mapwarn=FALSE,          # Faz um mapa com os pontos acima do LSC
                        ...)                    #Argumentos passados para outros metodos
                        
{
  
codbairro <- list(BT=94, CFP=c(54,55), DI=c(39,40), EM=73, MCP=c(51,52))
cod <- codbairro[[bairro]]
bairro.name <- switch(bairro,
                      BT="Brasilia Teimosa",
                      EM="Engenho do Meio",
                      DI="Dois Irmos",
                      CPF="Casa Forte",
                      MCP="Morro da Conceicao")

method <- match.arg(method)

if(length(obj$resp[[bairro]][[data]])==0)stop("\nNao ha observacoes na data especificada")

if(log){
      .resp <- log(obj$resp[[bairro]][[data]]$NRO_OVOS[!is.na(obj$resp[[bairro]][[data]]$NRO_OVOS)]+1)
      maxi <- log(max(obj$dados$NRO_OVOS, na.rm=T))
    }else{
      .resp <- obj$resp[[bairro]][[data]]$NRO_OVOS[!is.na(obj$resp[[bairro]][[data]]$NRO_OVOS)]  
      maxi <- max(obj$dados$NRO_OVOS, na.rm=T)
}

x <- coordinates(obj$pts[obj$pts$ID %in% unique(obj$resp[[bairro]][[data]]$COD_ARMADILHA)
                         [!is.na(obj$resp[[bairro]][[data]]$NRO_OVOS)],])[,1]
y <- coordinates(obj$pts[obj$pts$ID %in% unique(obj$resp[[bairro]][[data]]$COD_ARMADILHA)
                         [!is.na(obj$resp[[bairro]][[data]]$NRO_OVOS)],])[,2]

# Gerando o Grid #
boxbairro <- bbox(do.call("rbind", obj$poly[cod]))
gx <- seq(boxbairro[1], boxbairro[3], l=grau)
gy <- seq(boxbairro[2], boxbairro[4],l=grau)
grd <- expand.grid(x=gx, y=gy)
grd1 <- cbind(x, y)
#Borda
borda <- do.call("rbind", obj$poly[cod])


if(length(.resp)<19){
    formula<-as.formula(.resp~s(x, y, k=length(.resp)))
  }else{
    formula<-as.formula(.resp~s(x, k=5, ...)+s(y, k=5, ...))
  }

ajust <- switch(method, 
	gam=mgcv::gam(formula),
	loess=loess(.resp ~ x+y, control=loess.control(surface="direct"),...),
	akima=interp.new(x,y,.resp,
              xo=seq(min(x), max(x), length = grau),
              yo=seq(min(y), max(y), length = grau),
              linear=FALSE, extrap=TRUE ,
              duplicate="error", dupfun=NULL , ncp=NULL),
        tps=Tps(as.data.frame(cbind(x, y)), .resp, ...),
        mba=mba.surf(cbind(x, y, .resp),grau, grau, extend=T, ...)$xyz.est ,
        lm=lm(.resp ~ poly(x,2) + poly(y,2), ...),
        glm=glm(.resp ~ x+y,...)
)


if(method=="gam"){
	pred <- matrix(predict(ajust,grd),grau,grau)
	pred[!.geoR_inout(grd,borda)]<- NA
        out <- cbind(.resp, exp(predict(ajust, as.data.frame(grd1))))}
if(method=="loess"){
	pred <- predict(ajust, grd)
	pred[!.geoR_inout(grd, borda)] <- NA
  out <- cbind(.resp, predict(ajust, as.data.frame(grd1)))}
if(method=="akima"){
	pred <- ajust$z
	pred[!.geoR_inout(grd, borda)] <- NA}
if(method=="tps"){
  grid.l<- list(x=gx, y=gy)
  pred<-predict.surface(ajust, extrap=T, grid.list=grid.l)
  pred$z[!.geoR_inout(grd, borda)] <- NA
  pred <- pred$z}
if(method=="mba"){
  pred<-ajust$z
  pred[!.geoR_inout(grd, borda)] <- NA}
if(method=="lm"){
  pred <- predict(ajust, grd)
  dim(pred) <- c(grau,grau) 
  pred[!.geoR_inout(grd, borda)] <- NA
  out <- cbind(.resp, predict(ajust, as.data.frame(grd1)))}
if(method=="glm"){
  pred <- predict(ajust, grd)
  dim(pred) <- c(grau, grau)
  pred[!.geoR_inout(grd, borda)] <- NA
  out <- cbind(.resp, exp(predict(ajust, as.data.frame(grd1))))}

if(trunc){
  pred[which(pred>maxi)] <- maxi
  pred[which(pred<0)] <- 0
}
if(mapwarn){
  lsc <- ifelse(log==T,media[1] + qnorm(conf)*desv[1], media[2] + qnorm(conf)*desv[2])
  pred[which(pred < lsc)] <- 0
  pred[which(pred > lsc)] <- 1
}

n <- 18

if(plot){
  def.par <- par(no.readonly = TRUE)
  on.exit(par(def.par))
  mat <- matrix(1:n, ncol=n)
  if(!showTitle){
    nf <- layout(matrix(1:2, ncol=2),5, 1)
    par(mar=c(5.1,2,3,2))
  }else{
    nf <- layout(matrix(c(1,1,2,3), ncol=2, byrow=T), width=c(5,1), hei=c(1,8))
    par(mar=c(0.1,0.1,0.1,0.1))
    ltitle(paste("Metodo: ", method,"\nData: ", names(obj$resp[[bairro]][data]), 
                 "\nBairro: ", bairro.name, sep=""),ypos=1, cex=cext)
    box()
  }
  lab <- round(seq(min(pred, na.rm=T), max(pred, na.rm=T), l=n/2),2)  
  if(showTitle)par(mar=c(5.1,0.1,0.1,3))
     image(gx,gy,pred, asp=1,axes=F, xlab="", ylab="", main="", col=col)
	   lines(borda,lwd=2)
     if(points)points(x, y, pch="+")
     if(contour)contour(gx, gy, pred, add=T, lwd=1.5)
  if(showTitle) par(mar=c(5.1,1.2,0.1,4)) else par(mar=c(5.1,1,3,3.1))
	   image(mat, axes=F, col=col)
	   box()
     axis(4, at=seq(0,1, l=n/2), labels=lab, font=2, las=TRUE, cex=0.5)
}else{
  if(warn){
    if(!trunc)warning("Argumento trunc  Falso")
    lsc <- ifelse(log==T,media[1] + qnorm(conf)*desv[1], media[2] + qnorm(conf)*desv[2])
    cont <- length(pred[which(pred>lsc)])
    perc <- cont/length(which(pred>0))
    return(perc)
  }


  if(statistics){
    ret <- switch(method,
                  gam=list(Model=ajust, AIC=AIC(ajust), Residuals=out),
                  glm=list(Model=ajust, AIC=AIC(ajust), Residuals=out),
                  lm=list(Model=ajust, AIC=AIC(ajust), Residuals=out),
                  loess=list(Residuals=out))
    return(ret) 
  }else{
       return(structure(list(x=gx, y=gy, z=structure(pred, .Dim=c(grau, grau)),
                      coord.x=x, coord.y=y,Bairro=bairro, Metodo=method,Data=data),
                      .Names = c("x","y", "z", "coord.x", "coord.y", "Bairro", "Metodo","Data"),
                 class = "arma.pred"))
}
}
}

